import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/common.dart';
import 'package:wowit/sql_service.dart';

import 'methods.dart';

class Message {
  Message._();

  static const String prefMessage = 'prefMessage';
  static const String presUseMessageFromSever = 'messageSever';
  static const String prefLanguage = 'languageId';
  //get message by key
  static String getMessage(String key, {String? messageDefault}) {
    if (Common.mapMessageData.isEmpty) Common.setupDefaultData();
    if (!Common.canGetMessage) return key;
    if (key.isEmpty) return key;

    key = key.trim().toUpperCase();
    if (Common.showKeyMessage && Common.supperAdmin) {
      _setMessageBackUp(key, Common.mapMessageData[key]);
      return key;
    }

    if (Common.mapMessageData.containsKey(key)) {
      return Common.mapMessageData[key] ?? messageDefault ?? key.toUpperCase();
    }

    // not exists -> auto create SQL
    if (kDebugMode && Common.autoCreateMessage) {
      SQLService.execute('sp_SyncMessage', params: {
        'message': key,
        'messageId': key,
        'languageId': 1,
      });
    }
    if (kDebugMode && messageDefault != null && messageDefault.isNotEmpty) {
      messageDefault = '*$messageDefault';
    }
    Common.mapMessageData[key] = messageDefault ?? key;
    return messageDefault ?? key;
  }

  static String getMessageDynamic(String key, int viewID,{String? messageDefault,int? idComponentView}){
    String keyDynamic = 'DYNAMICVIEW_$viewID'  '_';
    if(idComponentView != null && idComponentView>0){
      keyDynamic = keyDynamic + '$idComponentView' '_' + key;
    }else {
      keyDynamic = keyDynamic  + key;
    }
    return getMessage(keyDynamic,messageDefault: messageDefault);
  }

  static void _setMessageBackUp(String key, String? message) {
    if (!Common.messageIdBackUp.containsKey(key)) {
      if (kDebugMode) {
        print('backUp key:$key, message: $message');
      }
      message = message ?? key.toUpperCase();
      Common.messageIdBackUp[key] = message;
    }
  }

  static Future setLanguage(int languageCode) async {
    var pref = await SharedPreferences.getInstance();
    Common.languageId = languageCode;
    await pref.setInt(prefLanguage, Common.languageId);
    await getMessageSQL(reload: true);
  }

  static Future<void> getMessageSQL({
    List<Map<String, Object?>>? dtLanguage,
    bool reload = false,
  }) async {
    //cong key kho vao, de dam bao khong gap ky tu user su dung.
    const pattern = '[(=-|';
    SharedPreferences prefs = await SharedPreferences.getInstance();
    int language = prefs.getInt(prefLanguage) ?? Common.languageId;
    Map<String, String> valueMessage = {};
    List<String> localMessage = prefs.getStringList(prefMessage) ?? [];
    if (localMessage.isEmpty || reload) {
      List<String> listLanguage = [];
      if (localMessage.isEmpty ||
          dtLanguage == null ||
          dtLanguage.isEmpty ||
          reload) {
        var data = await SQLService.execute('getMessage',
            params: {'@languageId': language});
        dtLanguage = Methods.getList(data, 'language');
      }

      await SQLService.execute('sp_setHasMapMessage',
          params: {'@uid': Common.uid});

      for (var item in dtLanguage) {
        listLanguage.add(Methods.getString(item, 'messageId') +
            pattern +
            Methods.getString(item, 'message'));
      }
      if (prefs.getStringList(prefMessage) != null) {
        await prefs.remove(prefMessage);
      }
      await prefs.setStringList(prefMessage, listLanguage);
      List<String> list = prefs.getStringList(prefMessage) ?? [];
      for (var item in list) {
        var convert = item.split(pattern);
        if (convert.length > 1) {
          String messageId = convert[0].toString().toUpperCase();
          if (!valueMessage.containsKey(messageId)) {
            valueMessage[messageId] = convert[1];
          }
        }
      }
      if (kDebugMode) {
        print('load Message from server success');
      }
    } else {
      List<String> list = localMessage;
      for (var item in list) {
        var convert = item.split(pattern);
        if (convert.length > 1) {
          String messageId = convert[0].toString().toUpperCase();
          if (!valueMessage.containsKey(messageId)) {
            valueMessage[messageId] = convert[1];
          }
        }
      }
      if (kDebugMode) {
        print('load Message from local success');
      }
    }
    Common.mapMessageData = valueMessage;
  }
}
